@@ -44,10 +44,10 @@
 (define (open-server-socket file-name)
   "Open a socket at FILE-NAME, and listen for connections there."
   (with-fluids ((%default-port-encoding "UTF-8"))
-    (let ((sock    (socket PF_UNIX SOCK_STREAM 0))
+    (let ((sock    (socket PF_UNIX
+                           (logior SOCK_STREAM SOCK_NONBLOCK SOCK_CLOEXEC)
+                           0))
           (address (make-socket-address AF_UNIX file-name)))
-      (fcntl sock F_SETFL (logior O_NONBLOCK
-                                  (fcntl sock F_GETFL)))
       (bind sock address)
       (listen sock 10)
       sock)))
@@ -151,8 +151,28 @@ already ~a threads running, disabling 'signalfd' support")
     ((signal-handler signal))))
 
 
+(define (mark-as-close-on-exec)
+  "Mark all the open file descriptors as close-on-exec."
+  (define max-fd
+    (max-file-descriptors))
+
+  (let loop ((fd 3))
+    (when (< fd max-fd)
+      (catch-system-error
+       (let ((flags (fcntl fd F_GETFD)))
+         (when (zero? (logand flags FD_CLOEXEC))
+           (fcntl fd F_SETFD (logior FD_CLOEXEC flags)))))
+      (loop (+ fd 1)))))
+
 (define* (run-daemon #:key (config-file (default-config-file)) persistency
                      socket-file pid-file signal-port poll-services?)
+  ;; We might have file descriptors inherited from our parent, as well as file
+  ;; descriptors wrongfully opened by Guile or Fibers (see
+  ;; <https://bugs.gnu.org/57567> and
+  ;; <https://github.com/wingo/fibers/commit/1f834cb81126dea2fd47d3d7ebb2d21f798a3c8b>);
+  ;; mark them all as FD_CLOEXEC so child processes do not inherit them.
+  (mark-as-close-on-exec)
+
   ;; This _must_ succeed.  (We could also put the `catch' around
   ;; `main', but it is often useful to get the backtrace, and
   ;; `caught-error' does not do this yet.)
@@ -213,9 +233,9 @@ already ~a threads running, disabling 'signalfd' support")
 
          ;; Enter some sort of a REPL for commands.
          (let next-command ()
-           (match (accept sock)
+           (match (accept sock (logior SOCK_NONBLOCK SOCK_CLOEXEC))
              ((command-source . client-address)
-              (setvbuf command-source (buffering-mode block) 1024)
+              (setvbuf command-source 'block 1024)
               (spawn-fiber
                (lambda ()
                  (process-connection command-source))))
@@ -251,6 +271,10 @@ already ~a threads running, disabling 'signalfd' support")
     ;; the signal thread.
     (maybe-signal-port %precious-signals))
 
+  (define log-flags
+    ;; Flags for 'open' when opening the log file.
+    (logior O_CREAT O_APPEND O_WRONLY O_CLOEXEC))
+
   (initialize-cli)
 
   (let ((config-file #f)
@@ -333,11 +357,13 @@ already ~a threads running, disabling 'signalfd' support")
     ;; Enable logging as first action.
     (parameterize ((log-output-port
                     (cond (logfile
-                           (open-file logfile "al"))
+                           (buffering (open logfile log-flags)
+                                      'line))
                           ((zero? (getuid))
                            (syslog-output-port))
                           (else
-                           (open-file (user-default-log-file) "al"))))
+                           (buffering (open (user-default-log-file) log-flags)
+                                      'line))))
                    (%current-logfile-date-format
                     (if (and (not logfile) (zero? (getuid)))
                         (format #f "shepherd[~d]: " (getpid))
