@@ -1,5 +1,5 @@
 ;; comm.scm -- Communication between processes and general output.
-;; Copyright (C) 2013, 2014, 2016, 2018, 2019 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2014, 2016, 2018, 2019, 2022 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2018 Danny Milosavljevic <dannym@scratchpost.org>
 ;;
@@ -83,7 +83,7 @@ return the socket."
       (catch 'system-error
         (lambda ()
           (connect sock address)
-          (setvbuf sock (buffering-mode block) 1024))
+          (setvbuf sock 'block 1024))
         (lambda (key proc format-string format-args errno . rest)
           ;; Guile's 'connect' throws an exception that doesn't specify
           ;; FILE.  Augment it with this information.
@@ -238,11 +238,13 @@ mechanism."
                 (apply throw args)))))
 
       (or (and port (not (port-closed? port)) (call/syslog))
-          (let ((sock (socket AF_UNIX SOCK_DGRAM 0)))
+          (let ((sock (socket AF_UNIX
+                              (logior SOCK_CLOEXEC SOCK_DGRAM)
+                              0)))
             (catch 'system-error
               (lambda ()
                 (connect sock AF_UNIX "/dev/log")
-                (setvbuf sock (buffering-mode line))
+                (setvbuf sock 'line)
                 (set! port sock)
                 (call/syslog))
               (lambda args
@@ -253,14 +255,14 @@ mechanism."
                       (lambda ()
                         (call-with-output-file "/dev/kmsg"
                           (lambda (port)
-                            (setvbuf port (buffering-mode block))
+                            (setvbuf port 'block)
                             (proc port))))
                       (lambda args
                         (if (memv (system-error-errno args)
                                   (list ENOENT EACCES EPERM))
                             (call-with-output-file "/dev/console"
                               (lambda (port)
-                                (setvbuf port (buffering-mode none))
+                                (setvbuf port 'none)
                                 (proc port)))
                             (apply throw args))))
                     (apply throw args)))))))))
