@@ -876,10 +876,11 @@ daemon writing FILE is running in a separate PID namespace."
               (try-again)
               (apply throw args)))))))
 
-(define (service-file-logger file input)
-  "Return a thunk meant to run as a fiber that reads from INPUT and logs it to
-FILE."
-  (let* ((fd     (open-fdes file (logior O_CREAT O_WRONLY O_APPEND) #o640))
+(define (%service-file-logger file input)
+  "Like 'service-file-logger', but doesn't handle the case in which FILE does
+not exist."
+  (let* ((fd     (open-fdes file (logior O_CREAT O_WRONLY O_APPEND O_CLOEXEC)
+                            #o640))
          (output (fdopen fd "al")))
     (set-port-encoding! output "UTF-8")
     (set-port-conversion-strategy! output 'substitute)
@@ -897,6 +898,19 @@ FILE."
                  (format output "~a~a~%" prefix line)
                  (loop))))))))))
 
+(define (service-file-logger file input)
+  "Return a thunk meant to run as a fiber that reads from INPUT and logs it to
+FILE."
+  (catch 'system-error
+    (lambda ()
+      (%service-file-logger file input))
+    (lambda args
+      (if (= ENOENT (system-error-errno args))
+          (begin
+            (mkdir-p (dirname file))
+            (%service-file-logger file input))
+          (apply throw args)))))
+
 (define (service-builtin-logger command input)
   "Return a thunk meant to run as a fiber that reads from INPUT and logs to
 'log-output-port'."
@@ -990,6 +1004,7 @@ false."
              (dup2 1 2)
 
              ;; Make EXTRA-PORTS available starting from file descriptor 3.
+             ;; This clears their FD_CLOEXEC flag.
              (let loop ((fd    3)
                         (ports extra-ports))
                (match ports
@@ -1033,23 +1048,16 @@ false."
      (when file-creation-mask
        (umask file-creation-mask))
 
-     ;; Last, close all file descriptors.  Do that after shutting down the
-     ;; finalization thread since we will close its pipe, leading to
-     ;; "error in the finalization thread: Bad file descriptor".
-     (without-automatic-finalization
-      (let loop ((i (+ 3 (length extra-ports))))
-        (when (< i max-fd)
-          (catch-system-error (close-fdes i))
-          (loop (+ i 1))))
-
-      (catch 'system-error
-        (lambda ()
-          (apply execlp program program args))
-        (lambda args
-          (format (current-error-port)
-                  "exec of ~s failed: ~a~%"
-                  program (strerror (system-error-errno args)))
-          (primitive-exit 1))))))))
+     (catch 'system-error
+       (lambda ()
+         ;; File descriptors used internally are all marked as close-on-exec,
+         ;; so we can fearlessly go ahead.
+         (apply execlp program program args))
+       (lambda args
+         (format (current-error-port)
+                 "exec of ~s failed: ~a~%"
+                 program (strerror (system-error-errno args)))
+         (primitive-exit 1)))))))
 
 (define %precious-signals
   ;; Signals that the shepherd process handles.
@@ -1084,7 +1092,7 @@ environment variable used for systemd-style \"socket activation\"."
   ;; handler, which stops shepherd, is called.  To avoid this, block signals
   ;; so that the child process never executes those handlers.
   (with-blocked-signals %precious-signals
-    (match (pipe)
+    (match (pipe2 O_CLOEXEC)
       ((log-input . log-output)
        (let ((pid (primitive-fork)))
          (if (zero? pid)
@@ -1268,8 +1276,12 @@ permissions for its parent directory."
   (match endpoint
     (($ <endpoint> name address style backlog
                    owner group permissions)
-     (let* ((sock    (non-blocking-port
-                      (socket (sockaddr:fam address) style 0)))
+     ;; Make listening sockets SOCK_CLOEXEC: inetd-style services don't pass
+     ;; them to the child process, and systemd-style do pass them but call
+     ;; 'dup2' right before 'exec', thereby clearing this property.
+     (let* ((sock    (socket (sockaddr:fam address)
+                             (logior SOCK_NONBLOCK SOCK_CLOEXEC style)
+                             0))
             (owner   (if (integer? owner)
                          owner
                          (passwd:uid (getpwnam owner))))
@@ -1500,7 +1512,7 @@ The remaining arguments are as for @code{make-forkexec-constructor}."
     ;; Return a thunk that accepts client connections from SOCK.
     (lambda ()
       (let loop ()
-        (match (accept sock)
+        (match (accept sock SOCK_CLOEXEC)
           ((connection . client-address)
            (if (>= connection-count max-connections)
                (begin
