@@ -24,12 +24,10 @@
   #:use-module (shepherd config)
   #:use-module (ice-9 match)
   #:use-module (ice-9 format)
-  #:export (buffering-mode
-
-            caught-error
+  #:export (caught-error
             assert
-            label
 
+            buffering
             catch-system-error
             with-system-error-handling
             with-atomic-file-output
@@ -68,14 +66,6 @@
 
             verify-dir))
 
-(define-syntax buffering-mode
-  (syntax-rules (line block none)
-    "Return the appropriate buffering mode depending on whether we're on Guile
-2.0 or later."
-    ((_ line) 'line)
-    ((_ block) 'block)
-    ((_ none) 'none)))
-
 ;; Report the caught error.
 ;; FIXME: Needs some more work.
 (define (caught-error key args)
@@ -98,11 +88,10 @@
 	 (local-output (l10n "Assertion ~a failed.") 'EXPR)
 	 (throw 'assertion-failed))))
 
-;; Recursive procedures.
-(define-syntax-rule (label NAME PROC)
-  (lambda args
-    (letrec ((NAME PROC))
-      (apply NAME args))))
+(define (buffering port type . args)
+  "Return PORT after changing its buffering to TYPE and ARGS."
+  (apply setvbuf port type args)
+  port)
 
 ;; Evaluate `EXPR ...' until a system error occurs, then skip the
 ;; remaining code.
@@ -204,8 +193,8 @@ output port, and PROC's result is returned."
 
   (bindtextdomain %gettext-domain %localedir)
   (textdomain %gettext-domain)
-  (setvbuf (current-output-port) (buffering-mode line))
-  (setvbuf (current-error-port) (buffering-mode line)))
+  (setvbuf (current-output-port) 'line)
+  (setvbuf (current-error-port) 'line))
 
 ;; Localized version of STR.
 (define-syntax l10n
